#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Traitement des catégories
# 
# Ce notebook présente différentes options pour gérer les catégories au format entier ou texte.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# On construit un jeu très simple avec deux catégories, une entière, une au format texte.

# In[3]:


import pandas
import numpy
df = pandas.DataFrame(dict(cat_int=[10, 20, 10, 39, 10, 10, numpy.nan],
                          cat_text=['catA', 'catB', 'catA', 'catDD', 'catB', numpy.nan, 'catB']))
df


# ## Transformations d'une catégorie
# 
# Les premières opérations consiste à convertir une catégorie au format entier ou au format texte en un entier. Les valeurs manquantes ne sont toujours traitées de la même façon.

# In[4]:


from sklearn.preprocessing import LabelEncoder
LabelEncoder().fit_transform(df['cat_int'])


# In[5]:


try:
    LabelEncoder().fit_transform(df['cat_text'])
except Exception as e:
    print(e)
LabelEncoder().fit_transform(df['cat_text'].dropna())


# On peut récupérer l'association entre catégorie et catégorie codée.

# In[6]:


le = LabelEncoder()
le.fit(df['cat_text'].dropna())
le.classes_


# La seconde opération permet de transformer une catégorie au format entier en plusieurs colonnes au format binaire.

# In[7]:


from sklearn.preprocessing import OneHotEncoder
try:
    OneHotEncoder().fit_transform(df[['cat_int']]).todense()
except Exception as e:
    print(e)
OneHotEncoder().fit_transform(df[['cat_int']].dropna()).todense()


# In[8]:


from sklearn.preprocessing import LabelBinarizer
try:
    LabelBinarizer().fit_transform(df[['cat_int']])
except Exception as e:
    print(e)
LabelBinarizer().fit_transform(df[['cat_int']].dropna())


# In[9]:


from sklearn.preprocessing import LabelBinarizer
try:
    LabelBinarizer().fit_transform(df[['cat_text']])
except Exception as e:
    print(e)
LabelBinarizer().fit_transform(df[['cat_text']].dropna())


# D'autres options qui ne fonctionnent pas tout à fait de la même manière en terme d'implémentation.

# In[10]:


from sklearn.feature_extraction import DictVectorizer
DictVectorizer().fit_transform(df.to_dict('records')).todense()


# In[11]:


from sklearn.feature_extraction import FeatureHasher
FeatureHasher(n_features=5).fit_transform(df.to_dict('records')).todense()


# ## Méthodes à gradient et ensemblistes
# 
# On construit un simple jeu de données pour une régression linéaire, $Y = -10 X - 7$ puis on cale une régression linéaire avec $Y \sim \alpha X_2 + \epsilon$ où $X_2$ est une permutation de *X*. Le lien est en quelque sorte brisé.

# In[12]:


perm = numpy.random.permutation(list(range(10)))
n = 1000
X1 = numpy.random.randint(0, 10, (n,1))
X2 = numpy.array([perm[i] for i in X1])
eps = numpy.random.random((n, 1))
Y = X1 * (-10) - 7 + eps
data = pandas.DataFrame(dict(X1=X1.ravel(), X2=X2.ravel(), Y=Y.ravel()))
data.head()


# In[13]:


from sklearn.model_selection import train_test_split
data_train, data_test = train_test_split(data)
data_train = data_train.copy()
data_test = data_test.copy()


# On transforme la catégorie :

# In[14]:


le = LabelEncoder().fit(data_train['X2'])
data_train['X3'] = le.transform(data_train['X2'])
data_test['X3'] = le.transform(data_test['X2'])
data_train.head()


# In[15]:


data_train.corr()


# On cale une régression linéaire de $Y$ sur $X_3$ la catégorie encodée :

# In[16]:


from sklearn.linear_model import LinearRegression
clr = LinearRegression()
clr.fit(data_train[['X3']], data_train['Y'])


# In[17]:


from sklearn.metrics import r2_score
r2_score(data_test['Y'], clr.predict(data_test[['X3']]))


# Autrement dit, elle n'a rien appris. On cale un arbre de décision :

# In[18]:


from sklearn.tree import DecisionTreeRegressor
clr = DecisionTreeRegressor()
clr.fit(data_train[['X3']], data_train['Y'])


# In[19]:


r2_score(data_test['Y'], clr.predict(data_test[['X3']]))


# L'arbre de décision a saisi la permutation alors que la régression linéaire n'a pas fonctionné. La régression linéaire n'est pas estimée à l'aide d'une méthode à base de gradient mais elle possède les mêmes contraintes, il est préférable que la cible *Y* soit une fonction le plus possible monotone de *X*. Avec une colonne par modalité de la catégorie, le résultat est tout autre.

# In[20]:


one = OneHotEncoder().fit(data_train[['X2']])
feat = one.transform(data_train[['X2']])
feat[:5].todense()


# In[21]:


clr = LinearRegression()
clr.fit(feat, data_train['Y'])


# In[22]:


r2_score(data_test['Y'], clr.predict(one.transform(data_test[['X2']])))


# In[23]:





# In[24]: